/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:    snmpAgent.c
 *
 * Date:        May 1994
 *
 * Purpose:     Agent program to test/demonstrate the useability of the 
 *		SNMPv2 API-functions. Agnt acts like a native-proxy Agnt.
 *
 * Author(s):   J.W.Togtema, Eric van Hengstum, Vincent Berkhout.
 *		Anton A(pi) Holleman. 
 *
 * -----------------------------------------------------------------------
 */

#include <apiSNMP.h>

#include <sys/types.h>
#include <sys/time.h>
#include <sys/uio.h>
#include <unistd.h>

#define AgntMain main
/*
 *      Definition of linked list for mapping corresponding requests.
 */
typedef struct outreq {
	int     responseid;
	int     requestid;
	struct outreq *next;
} OUTREQ;
OUTREQ  *outstreqlist = NULL;

/*
 *      Request sequence number.
 */
INTEGER REQ_SEQNR = 5000;

/*
 *      Must this application produce debug output? Default is no.
 */
int AgntDEBUG = 0;


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntErrorTypes
 *
 * Arguments:	
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:    
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntErrorTypes( INTEGER errorstatus )
{
	switch( (int) errorstatus )  {
	case 0 : fprintf( stdout, "noError"); break;
	case TOOBIG : fprintf( stdout, "tooBig"); break;
	case NOSUCHNAME : fprintf( stdout, "noSuchName"); break;
	case BADVALUE : fprintf( stdout, "badValue"); break;
	case READONLY : fprintf( stdout, "readOnly"); break;
	case GENERR : fprintf( stdout, "genErr"); break;
	case NOACCESS : fprintf( stdout, "noAccess"); break;
	case WRONGTYPE : fprintf( stdout, "wrongType"); break;
	case WRONGLENGTH : fprintf( stdout, "wrongLength"); break;
	case WRONGENCODING : fprintf( stdout, "wrongEncoding"); break;
	case WRONGVALUE : fprintf( stdout, "wrongValue"); break;
	case NOCREATION : fprintf( stdout, "noCreation"); break;
	case INCONSISTENTVALUE : fprintf( stdout, "inconsistentValue"); break;
	case RESOURCEUNAVAILABLE : fprintf( stdout, "resourceUnavailable"); break;
	case COMMITFAILED : fprintf( stdout, "commitFailed"); break;
	case UNDOFAILED : fprintf( stdout, "undoFailed"); break;
	case AUTHORIZATIONERROR : fprintf( stdout, "authorizationError"); break;
	case NOTWRITABLE : fprintf( stdout, "notWritable"); break;
	case INCONSISTENTNAME : fprintf( stdout, "inconsistentname"); break;
	default: fprintf (stdout,"unknownError: %d",errorstatus);
	}
	return OK ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntIdIncrement
 *
 * Arguments:	id, request id (IN/OUT).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Produce a new request identifier and assign it to the 
 *		parameter 'id'.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntIdIncrement( INTEGER *id )
{
	*id = REQ_SEQNR;
	REQ_SEQNR++;
	if ( REQ_SEQNR == 10000 )
		REQ_SEQNR = 0;
	return( 0 );
}	


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntDeleteEntryList
 *
 * Arguments:	entry, pointer to entry to be deleted/removed (IN).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     unlink entry 'entry' from the global linked list 
 *		'outstreqlist' and free the alloced memory of 'entry'.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntDeleteEntryList( OUTREQ *entry )
{
	OUTREQ *curr;

	curr = outstreqlist; 
	if( curr == (OUTREQ *) NULL )
		return( -1 );
	if( entry == curr )
	{
		outstreqlist = curr->next;
		(void) free( curr );
		return( 0 );
	}
	while( curr->next != (OUTREQ *) NULL )
	{
	       	if( curr->next == entry)
		{
		       	curr->next = entry->next;
		       	(void) free( entry );
			return( 0 );
		} else
			curr = curr->next;
	}
	return( -1 );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntCreateEntryList
 *
 * Arguments:	list, newly allocated entry pointed by list (IN/OUT).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Allocate memory for new entry 'list' and prepend entry
 *		on top of global linked-list 'outstreqlist'.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntCreateEntryList( OUTREQ **list )
{
	*list = (OUTREQ *) malloc( sizeof( OUTREQ ));
	if( *list == (OUTREQ *) NULL )
		return( -1 );
	(*list)->next = outstreqlist; 
	outstreqlist = *list;
	return( 0 );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntgetEntryOutList
 *
 * Arguments:	requestid, identifier of incoming communication (IN).
 * 		reponseid, identifier of outgoing communication (OUT).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Search for 'requestid' and it's corresponsing 'responseid'
 *		in global linked list 'outstreqlist'. Remove entry, iff
 *		found.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntgetEntryOutList( INTEGER requestid, INTEGER *responseid )
{
	OUTREQ	*curr = NULL;

	curr = outstreqlist;
	while( curr != NULL )
	{
		if( curr->requestid == requestid )
		{
			*responseid = curr->responseid;			
			(void) AgntDeleteEntryList( curr );
			return( 0 );
		} else 
			curr = curr->next;
	}
	return( -1 );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntPutEntryInList
 *
 * Arguments:	responsid, identifier for outgoing communication (IN).
 *		requestid, identifier for incoming communication (IN).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Create an entry, fill entry with both values 'responseid' and 
 *		'requestid', and link the entry in the global linked-list 
 *		'outstreqlist'.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntPutEntryInList( INTEGER responseid, INTEGER requestid )
{
	OUTREQ *list;

	if( AgntCreateEntryList( &list ) )
		return( -1 );
	list->requestid = requestid; 
	list->responseid = responseid; 
	return( 0 );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntHandleNet
 *
 * Arguments:	type, management communication type(IN).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Routine handle all communication related to provider.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntHandleNet( SPTYPE type )
{
	INTEGER		toId = 0, fromId, ErrStat =0, ErrInd = 0, MaxRep, NonRep;
	NAME		fromAgnt="", toAgnt="utis94", toCnt="";
	QOS		toQoS=qosNONE;
	VarBindList	*Binds=mVlNew();
	VarBindList	*newvbl=mVlNew();
	VarBindList	*totalvbl =mVlNew(); 
	VarBind		*vb, *newvb;
	AsnOid		*newOid;
	VarBindChoice	*Ref_To_Value;
	int		n;
	DESCRIPTION	error;


        system( "clear" );
	StarLine; NewLine;
		
	switch( type ) {

	case GET_IND:
	ErrStat = 0;
	(void) snmpGetIndication( &fromId, &Binds );

/*	(void) AgntIdIncrement( &toId );
	(void) AgntPutEntryInList( fromId, toId );*/

	fprintf( stdout, "\tsnmpGetIndication(\tINTEGER replyId = %d,\n", fromId );
	fprintf( stdout, "\t\t\t\tVARLIST variableNames =\n" );
	(void) PrintVarBindList( stdout, Binds, 32 );
	fprintf( stdout, "\t);\n\n" );

	(void) snmpMetaGetRequest(fromId, Binds );

	fprintf( stdout, "\tsnmpMetaGetRequest(\tINTEGER replyId = %d,\n", fromId );
	fprintf( stdout, "\t\t\tVARLIST variableNames =\n" );
	(void) PrintVarBindList( stdout, Binds, 32 );
	fprintf( stdout, "\t);\n\n" );
	break;


	case META_GET_CNF:
	(void) snmpMetaGetConfirm( &fromId, &ErrStat, &ErrInd, &Binds );	
/*	(void) AgntgetEntryOutList( toId, &fromId );*/
	fprintf( stdout, "\tsnmpMetaGetConfirm(\tINTEGER replyId = %d,\n", fromId );
	fprintf( stdout, "\t\t\tINTEGER errorStatus = " );
	AgntErrorTypes( ErrStat );
	fprintf( stdout, ",\n" );
	fprintf( stdout, "\t\t\tINTEGER errorIndex = %d,\n", ErrInd );
	fprintf( stdout, "\t\t\tVARLIST variableValues =\n" );
	fprintf( stdout, "\t\t\t" );
	(void) PrintVarBindList( stdout, Binds, 24 ) ;
	fprintf( stdout, "\n" );
	fprintf( stdout, "\t);\n\n" );
	
	(void) snmpGetResponse( fromId, ErrStat, ErrInd, Binds );
	fprintf( stdout, "\tsnmpGetResponse(\tINTEGER replyId = %d,\n", fromId );
	fprintf( stdout, "\t\t\t\tINTEGER errorStatus = " );
	AgntErrorTypes( ErrStat );
	fprintf( stdout, ",\n" );
	fprintf( stdout, "\t\t\t\tINTEGER errorIndex = %d,\n", ErrInd );
	fprintf( stdout, "\t\t\t\tVARLIST variableValues =\n" );
	fprintf( stdout, "\t\t\t" );
	(void) PrintVarBindList( stdout, Binds, 24 ) ;
	fprintf( stdout, "\n" );
	fprintf( stdout, "\t);\n\n" );
	break;


	case GETNEXT_IND:
	ErrStat = 0;
	(void) snmpGetNextIndication( &fromId, &Binds );
/*	(void) AgntIdIncrement( &toId );
	(void) AgntPutEntryInList( fromId, toId );*/
	fprintf( stdout, "\tsnmpGetNextIndication(\tINTEGER replyId = %d,\n", fromId );
	fprintf( stdout, "\t\t\t\tVARLIST variableNames =\n" );
	(void) PrintVarBindList( stdout, Binds, 3) ;
	fprintf( stdout, "\t);\n\n" );
	(void) snmpMetaGetNextRequest(fromId, Binds);
	fprintf( stdout,"\tsnmpMetaGetNextRequest(\tINTEGER replyid = %d)\n",fromId);
	fprintf( stdout, "\t\t\t\tVARLIST variableNames =\n" );
	(void) PrintVarBindList( stdout, Binds, 3) ;
	fprintf( stdout, "\t);\n\n" );
	break;	
	
	case META_GETNEXT_CNF:
	(void) snmpMetaGetNextConfirm( &fromId, &ErrStat, &ErrInd, &Binds );
/*	(void) AgntgetEntryOutList( toId, &fromId );*/
	fprintf( stdout, "\tsnmpMetaGetNextConfirm(\tINTEGER replyId = %d,\n", fromId );
	fprintf( stdout, "\t\t\t\tINTEGER errorStatus = " );
	AgntErrorTypes( ErrStat );
	fprintf( stdout, ",\n" );
	fprintf( stdout, "\t\t\t\tINTEGER errorIndex = %d,\n", ErrInd );
	fprintf( stdout, "\t\t\t\tVARLIST variableValues =\n" );
	(void) PrintVarBindList( stdout, Binds, 24) ;
	fprintf( stdout, "\t\t\t" );
	fprintf( stdout, "\t);\n\n" );
	(void) snmpGetNextResponse( fromId, ErrStat, ErrInd, Binds );
	fprintf( stdout, "\tsnmpGetNextResponse(\tINTEGER replyId = %d,\n", fromId );
	fprintf( stdout, "\t\t\t\tINTEGER errorStatus = " );
	AgntErrorTypes( ErrStat );
	fprintf( stdout, ",\n" );
	fprintf( stdout, "\t\t\t\tINTEGER errorIndex = %d,\n", ErrInd );
	fprintf( stdout, "\t\t\t\tVARLIST variableValues =\n" );
	(void) PrintVarBindList( stdout, Binds, 24) ;
	fprintf( stdout, "\t);\n\n" );
	break;

	case GETBULK_IND:
	ErrStat = 0;
	(void) snmpGetBulkIndication( &fromId, &MaxRep, &NonRep, &Binds );
	fprintf( stdout, "\tsnmpGetBulkIndication(\tINTEGER replyId = %d,\n", fromId );
	fprintf( stdout, "\t\t\t\tINTEGER maxRepetitions = %d,\n", MaxRep );
	fprintf( stdout, "\t\t\t\tINTEGER maxRepeaters = %d,\n", NonRep );
	fprintf( stdout, "\t\t\t\tVARLIST variableNames =\n" );
	fprintf( stdout, "\t\t\t\t" );
	(void) PrintVarBindList( stdout, Binds, 32 ) ;
	fprintf( stdout, "\n" );
	fprintf( stdout, "\t);\n\n" );
	
	(void) snmpMetaGetNextRequest(fromId, Binds );

	pFreeVbl(Binds);
	while (snmpLook(&type) != OK) {
	}
	if (type == META_GETNEXT_CNF) {
		(void) snmpMetaGetNextConfirm( &fromId, &ErrStat, &ErrInd, &Binds );
	}
	n = 1;
	FOR_EACH_LIST_ELMT (vb,Binds) {
		mVlPut( totalvbl, vb);
		if ((n++) > NonRep) {
			Ref_To_Value = mVbGet (vb,newOid=mOiNew(255));
			newvb = mVbPut(mVbNew(255),newOid,mVcPut(mVcNew(),VARBINDCHOICE_UNSPECIFIED,NULL));
			mVlPut( newvbl, newvb);
		}
	}
	n = 1;
	while ((n++) < MaxRep) {
		(void) snmpMetaGetNextRequest(fromId, newvbl );
/*		fprintf( stdout, "\tsnmpMetaGetNextRequest(\tNAME host = %d,\n", fromId );
		fprintf( stdout, "\t\t\tVARLIST variableNames =\n" );
		fprintf( stdout, "\t\t\t\t" );
		(void) PrintVarBindList( stdout, newvbl, 32 ) ;
		fprintf( stdout, "\n" );
		fprintf( stdout, "\t);\n\n" );
*/		
		newvbl->first = NULL;
		mVlFree(newvbl);
		newvbl=mVlNew();
		while (snmpLook(&type) != OK) {
		}
		if (type == META_GETNEXT_CNF) {
			(void) snmpMetaGetNextConfirm( &fromId, &ErrStat, &ErrInd, &newvbl );
/*			fprintf( stdout, "\tsnmpMetaGetNextConfirm(\tINTEGER replyId = %d,\n", fromId );
			fprintf( stdout, "\t\t\t\tINTEGER errorStatus = " );
			AgntErrorTypes( ErrStat );
			fprintf( stdout, ",\n" );
			fprintf( stdout, "\t\t\t\tINTEGER errorIndex = %d,\n", ErrInd );
			fprintf( stdout, "\t\t\t\tVARLIST variableValues =\n" );
			(void) PrintVarBindList( stdout, newvbl, 24) ;
			fprintf( stdout, "\t\t\t" );
			fprintf( stdout, "\t);\n\n" );
*/
		}
		FOR_EACH_LIST_ELMT (vb,newvbl) {
			mVlPut( totalvbl, vb);	
		}
	}
	(void) snmpGetBulkResponse( fromId, ErrStat, ErrInd, totalvbl);
	fprintf( stdout, "\tsnmpGetBulkConfirm(\n" );
	fprintf( stdout, "\t\t\tINTEGER replyId = %d,\n", fromId );
	fprintf( stdout, "\t\t\tINTEGER errorStatus = " );
	AgntErrorTypes( ErrStat );
	fprintf( stdout, ",\n" );
	fprintf( stdout, "\t\t\tINTEGER errorIndex = %d,\n", ErrInd );
	fprintf( stdout, "\t\t\tVARLIST variableValues =\n" );
	fprintf( stdout, "\t\t\t" );
	(void) PrintVarBindList( stdout, totalvbl, 24 ) ;
	fprintf( stdout, "\n" );
	fprintf( stdout, "\t);\n\n" );
	break;


	case SET_IND:
	ErrStat = 0;
	(void) snmpSetIndication( &fromId, &Binds );
/*	(void) AgntIdIncrement( &toId );
	(void) AgntPutEntryInList( fromId, toId ); */
	fprintf( stdout, "\tsnmpSetIndication(\tINTEGER replyId = %d,\n", fromId );
	fprintf( stdout, "\t\t\t\tVARLIST variableValues =\n" );
	fprintf( stdout, "\t\t\t\t" );
	(void) PrintVarBindList( stdout, Binds, 32 ) ;
	fprintf( stdout, "\n" );
	fprintf( stdout, "\t);\n\n" );

	(void) snmpMetaSetRequest( fromId, Binds );
	fprintf( stdout, "\tsnmpMetaSetRequest(\tINTEGER replyId = %d,\n", fromId );
	fprintf( stdout, "\t\t\tVARLIST variableValues =\n" );
	fprintf( stdout, "\t\t\t\t" );
	(void) PrintVarBindList( stdout, Binds, 32 ) ;
	fprintf( stdout, "\n" );
	fprintf( stdout, "\t);\n\n" );
/*	break;  */

	
	while (snmpLook(&type) != OK) {
	}
	if (type == META_SET_CNF) {
/*	case META_SET_CNF:  */
	
	(void) snmpMetaSetConfirm( &fromId, &ErrStat, &ErrInd );
/*	(void) AgntgetEntryOutList( toId, &fromId );*/
	
	fprintf( stdout, "\tsnmpMetaSetConfirm(\tINTEGER replyId = %d,\n", fromId );
	fprintf( stdout, "\t\t\tINTEGER errorStatus = " );
	AgntErrorTypes( ErrStat );
	fprintf( stdout, ",\n" );
	fprintf( stdout, "\t\t\tINTEGER errorIndex = %d );\n\n", ErrInd );
	fprintf( stdout, "\t\t\t\tVARLIST variableValues =\n" );
	fprintf( stdout, "\t\t\t\t" );
	(void) PrintVarBindList( stdout, Binds, 32 ) ;
	fprintf( stdout, "\n" );
	fprintf( stdout, "\t);\n\n" );

	(void) snmpMetaCommitRequest (fromId);
	fprintf( stdout, "\tsnmpMetaCommitRequest(\tINTEGER replyId = %d,\n", fromId );

	while (snmpLook(&type) != OK) {
	}
	if (type == META_COMMIT_CNF) {
	(void) snmpMetaCommitConfirm(&fromId, &ErrStat);
	fprintf( stdout, "\tsnmpMetaCommitConfirm(\tINTEGER replyId = %d,\n", fromId );
	fprintf( stdout, "\t\t\tINTEGER errorStatus = " );
	AgntErrorTypes( ErrStat );

	(void) snmpSetResponse( fromId, ErrStat, ErrInd );
	fprintf( stdout, "\tsnmpSetResponse(\tINTEGER replyId = %d,\n", fromId );
	fprintf( stdout, "\t\t\tINTEGER errorStatus = " );
	AgntErrorTypes( ErrStat );
	fprintf( stdout, ",\n" );
	fprintf( stdout, "\t\t\tINTEGER errorIndex = %d );\n\n", ErrInd );
	fprintf( stdout, "\t\t\t\tVARLIST variableValues =\n" );
	fprintf( stdout, "\t\t\t\t" );
	(void) PrintVarBindList( stdout, Binds, 32 ) ;
	fprintf( stdout, "\n" );
	fprintf( stdout, "\t);\n\n" );
	}
	}
	break;


	case ERR_IND:
	(void) snmpErrorDescription( &toId, error );
	(void) AgntgetEntryOutList( toId, &fromId );
	fprintf( stderr, "\tsnmpErrorDescription(\tINTEGER requestId = %d,\n", toId );
	fprintf( stderr, "\t\t\t\terror = %s\n", error );
	fprintf( stderr, "\t);\n" );
	break;


	case TRAP_IND:
	(void) snmpTrapIndication( fromAgnt, toCnt, &Binds );

	fprintf( stdout, "\tsnmpTrapIndication(\n" );
	fprintf( stdout, "\t\t\tNAME host = %s,\n", fromAgnt );
	fprintf( stdout, "\t\t\tNAME application = %s,\n", toCnt );
	fprintf( stdout, "\t\t\tVARLIST variableValues =\n" );
	fprintf( stdout, "\t\t\t" );
	(void) PrintVarBindList( stdout, Binds, 24) ;
	fprintf( stdout, "\n" );
	fprintf( stdout, "\t);\n\n" );
	break;


	case INFORM_IND:
	(void) snmpInformIndication( fromAgnt, toCnt, &Binds );

	fprintf( stdout, "\tsnmpInformIndication(\n" );
	fprintf( stdout, "\t\t\tNAME host = %s,\n", fromAgnt );
	fprintf( stdout, "\t\t\tNAME application = %s,\n", toCnt );
	fprintf( stdout, "\t\t\tVARLIST variableValues =\n" );
	fprintf( stdout, "\t\t\t" );
	(void) PrintVarBindList( stdout, Binds, 24) ;
	fprintf( stdout, "\n" );
	fprintf( stdout, "\t);\n\n" );
	break;


	default: 
		mVlFree( Binds );
		return OK ;
	}
	pFreeVbl( Binds );
	NewLine; StarLine; 
	fprintf( stdout,"\n\tHit <enter> key to exit: " ); 
	Flush; 
	return FALSE;
}


/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    AgntUserMonitor
 *
 * Arguments:
 *
 * Return:      STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Prints the results.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntUserMonitor( )
{
	fd_set fds;
	struct timeval tv;

	tv.tv_sec = 0;
	tv.tv_usec = 10;

	FD_ZERO( &fds );
	FD_SET( 0, &fds );
	 
	if( select( getdtablesize(), &fds, 0, 0, &tv ) > 0 )
		return( OK );
	return( FAILED );
}



/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    AgntHandleUser
 *
 * Arguments:
 *
 * Return:      STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function fetches the user input.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntHandleUser( )
{
	return TRUE;
}


/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    AgntUserIntro
 *
 * Arguments:   argv, argument vector (IN)
 *
 * Return:      STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function prints the intro screen.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntUserIntro( char** argv )
{
        system( "clear" ); StarLine;
	fprintf( stdout, " *\t\t\t\t\t\t\t\t\t*\n" );
	fprintf( stdout, " * %s (version %s)\t\t\t\t\t*\n", argv[0], VERSION );
	fprintf( stdout, " * ---------\t\t\t\t\t\t\t\t*\n" );
	fprintf( stdout, " *\t\t\t\t\t\t\t\t\t*\n" );
	fprintf( stdout, " * This proxy agent demonstrates/tests the UT-SNMP-V2_1" );
	fprintf( stdout, " package.\t*\n");
	fprintf( stdout, " *\t\t\t\t\t\t\t\t\t*\n" );
	fprintf( stdout, " *\t\t\t\t\tSuccess, \t\t\t*\n" );
	fprintf( stdout, " *\t\t\t\t\t\tsnmp@cs.utwente.nl\t*\n" );
	fprintf( stdout, " *\t\t\t\t\t\t\t\t\t*\n" );
	fprintf( stdout, " * Additional remarks:\t\t\t\t\t\t\t*\n");
	fprintf( stdout, " *\t1) %% setenv SNMP_PARTYMIB_PATH ./ \t# path for partyfiles\t*\n");
	fprintf( stdout, " *\t\t\t\t\t\t\t\t\t*\n" );
	StarLine;  NewLine; 
	fprintf( stdout,"\tHit <enter> key to exit: " ); 
	Flush;
	return OK ;
}


/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    AgntNetMonitor
 *
 * Arguments:   type, the arrived management communication type (OUT).
 *
 * Return:      STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function monitors the snmp layer .
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntNetMonitor( SPTYPE *type )
{
	return( snmpLook( type ) );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntMain
 *
 * Arguments:	argc, argument octetLen (IN).
 *		argv, argument vector (IN).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     main driver for dispatching the tasks depending on the 
 *		occurred events.		
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	void
main( int argc, char **argv, char **arge ) 
{
	SPTYPE type = 0;
	BOOLEAN	dying = FALSE;


	if( snmpOpen( "snmpMetaAgent", arge ) ) {
		Error( "couldn't open snmp" );
		exit( 0 );
	}

	(void) AgntUserIntro( argv );
		
	while( ! dying ) {
		if( AgntUserMonitor( ) == OK )
			dying = AgntHandleUser( );
		else if( AgntNetMonitor( &type ) == OK )
		  	dying = AgntHandleNet( type );
												  }

	if( snmpClose( GRACEFULL ) ) {
		Error( "couldn't close snmp" );
		exit( -1 );
	}
	exit( 0 );
}

/* -----------------------end of file snmpAgent.c -------------------------- */
