/* -----------------------------------------------------------------------
 *
 * Project:		UT-SNMP
 *
 * Filename: 	UDP.c
 *
 * Date:		July 1994
 *
 * Purpose:		implementation of transport level interface with sockets
 *
 * Author(s):	Vincent Berkhout, Frederic P. H. van Hengstum 
 *
 * -----------------------------------------------------------------------
 */

#include <simTF.UDP.h>

/*
 * 	local variables
 */
static BOOLEAN SNMP_DEBUG_SOCKET = FALSE;
int localHandle=-1;


/*------------------------------------------------------------------------*/
	STATUS
udpCheckInputs( allHandles, activeHandles ) 
int 	*allHandles,	/* in, all available handle/filedescriptors */
	*activeHandles;	/* in, all handles/filedescriptors that are active */ 
/* 
 * Purpose:
 *	This call checks whether data is ready to be received on the 
 *	given handles "allHandles". The handles for which data is 
 *	returned by "activeHandles".
 * Remarks:
 * 	STATUS returns nrActiveHandles on success!
 */
{	
	int	i, j, nrActiveHandles, maxFiles;	
	fd_set	readset;
	struct 	timeval	timeout;
	struct	rlimit	r ;	/* maximimum number of fildescriptors available */

/*
 *	initialize the readset and the empty time value
 */
	FD_ZERO( &readset ) ;
	timeout.tv_sec = 0 ;
	timeout.tv_usec = 10 ;
	(void) getrlimit( RLIMIT_NOFILE, &r );
	maxFiles = 32 ; /* r.rlim_cur ; */	
/* 
 * 	incorporate handles (integer) into readset ("filedescriptor set") 
 */
	for (i = 0; i < maxFiles ; i++) {
		if ( allHandles[i] == -1 ) 
			break;
		if ( allHandles[i] > 1024 )
            FD_SET( localHandle, &readset ) ;
		else
			FD_SET( allHandles[i] , &readset ) ;
	}	/* endfor i to maxFiles */
/* 
 * 	now perform the select
 */
	nrActiveHandles = select( maxFiles, (fd_set *)&readset, (fd_set *)NULL, (fd_set *)NULL, &timeout);

/* 
 * 	and fill the array of int (activeHandles) with the active handles
 *	(If a handle is active it is copied from allHandles to activeHandles)
 */
	if (nrActiveHandles <= 0)
		activeHandles[0] = -1 ;
	else
		for( i = 0, j = 0; i < maxFiles; i++ ) {
			if( ( allHandles[i] > 1024 ) && FD_ISSET( localHandle, &readset ) )
				activeHandles[j++] = allHandles[i] ;
			else if( ( allHandles[i] > -1 ) && FD_ISSET( allHandles[i], &readset ) )
				activeHandles[j++] = allHandles[i] ;
		}

	return nrActiveHandles ;

}	/* end of function  SocketCheckInputs */


/*------------------------------------------------------------------------*/
	STATUS
udpClose ( socket ) 
int 	socket;			/* in, filedescriptor to be closed */
/*
 * Purpose:
 * 	The call simply closes the socket/filedescriptor with the systemcall close
 * Remarks:
 */
{
	if ( close ( socket )  < 0) {
		return ERROR ;
	}	/* endif close  */
	return OK ;
}	/* end of function SocketClose */


/*------------------------------------------------------------------------*/
	STATUS
udpOpen( localTAddress, dataSocket) 
u_char 	*localTAddress;	/* in, the port on which to listen */
int	*dataSocket;	/* out, a socket on which data can be received */
/*
 * Purpose:
 * 	This call performs an open for udp at the local side . It derives the
 * 	portnumber from the localTAddress parameter. It subsequently creates
 *	a socket, binds it to the portnumber and goes into "listen" mode.
 * Remarks:
 */
{
	int	msgSize = PACKETSIZE ;
	long	hostNr ;
	short	portNr ;
	struct 	sockaddr_in	newSock ;
	
/* 
 * 	first  open a socket with some initial values
 */
	*dataSocket = socket( PF_INET,  SOCK_DGRAM, 0) ;
	if( *dataSocket < 0)   {
		return ERROR ;
	}	/* endif socket creation */
/*
 *	set the packetsize as big as possible, preferably at 64 K
 */ 
	if ( setsockopt(*dataSocket, SOL_SOCKET, SO_SNDBUF, (char *)&msgSize, sizeof(msgSize) ) < 0 ) {
	} 	/* endif set SNDBUF */
	if ( setsockopt(*dataSocket, SOL_SOCKET, SO_RCVBUF, (char *)&msgSize, sizeof(msgSize) ) < 0 ) {
	} 	/* endif set RCVBUF */

/*
 *	get the port number from transport address
 */
  	(void) memcpy( &(hostNr), (const void *) localTAddress,  4 );
	(void) memcpy( &(portNr), (const void *) (localTAddress + 4) , 2);
/*
 *	fill in the socket properties
 */
	newSock.sin_family = AF_INET ; 			/* it is an internet address */
	newSock.sin_addr.s_addr = htonl( hostNr ) ;		/* address wild card */
	newSock.sin_port = htons( portNr ) ; 		/* assign the port number*/
/* 
 * 	now bind the socket to the transport address which was just filled in
 */
	if (bind( *dataSocket , (struct sockaddr *)&newSock, sizeof( newSock ) ) < 0 ) {
		newSock.sin_port = htons( 0 ) ;
		if (bind( *dataSocket, (struct sockaddr *)&newSock, sizeof(newSock)) < 0 ) {
			(void) close (*dataSocket ) ;
			return ERROR  ;
		}
	}

	/* HACK begin */
	msgSize = sizeof( newSock );
	getsockname( *dataSocket, (struct sockaddr *) &newSock, &msgSize );
	localHandle=*dataSocket;
	*dataSocket=newSock.sin_port;
	/* HACK end */

	return OK ;
} 	/* end of function SocketPacketOpen */


/*------------------------------------------------------------------------*/
	STATUS	
udpSend( dataSocket, remoteTAddress, message)
int	*dataSocket;		/* in, the datasocket to send the data to */
u_char	*remoteTAddress;	/* in, the remote address to be incorparated in the message */
char	*message;		/* in, the data to be send*/
/* 
 * Purpose:
 * 	This function sends the data to the remote address. 
 * 	After conversion of the address, the messagesize is determined and the
 *	message is send. 
 * Remarks: 
 */
{
	int  	nrWritten,
		addressSize,
		messageSize;
	long	hostNr;
	short	portNr;
	struct	sockaddr_in 	remote;
/* 
 * 	first convert the transport address 
 */
	(void) memset( (char *)&remote, '\0', sizeof(struct sockaddr_in)) ;
  	(void) memcpy( &hostNr, (const void *)remoteTAddress, 4 );
	(void) memcpy( &portNr, (const void *)(remoteTAddress + 4), 2);
/*
 * 	fill in the remote address
 */
	remote.sin_family = AF_INET;   		/* tcp belongs to the internet address family*/
	remote.sin_addr.s_addr = htonl(hostNr);	/* convert hostNr to network byte order */
	if( *dataSocket > 1024 )
		portNr=*dataSocket;
	remote.sin_port = htons(portNr);
	(void) memcpy( &messageSize, message, 4 );
	messageSize=messageSize+4;
/* 
 * 	and finally send the data 
 */
	addressSize = sizeof (remote) ;

	if( *dataSocket > 1024 )
		*dataSocket=localHandle;

	nrWritten = sendto( *dataSocket, message, messageSize, 0, 
						(struct sockaddr *) &remote, addressSize );

	if( nrWritten < 0 ) {
		return FAILED ;
	}	/* end if nrWritten */
	return OK ;
} 	/* end of function SocketStreamSend */


/*------------------------------------------------------------------------*/
	STATUS	
udpReceive( dataSocket, remoteTAddress, message)
int	*dataSocket; 		/* in, socket to receive on  */
u_char	*remoteTAddress;  	/* out, remote address of the sending peer */
char	*message;		/* out, the data that has been send by the peer */
/*
 * Purpose:
 *	Reads the data from the given socket and returns the message AND peer address
 * Remarks:
 */
{	
	int  	nrReads,
		addressSize,
		msgSize = PACKETSIZE ;
	long	hostNr ;
	short	portNr ;
	struct	sockaddr_in	remote;

	addressSize = sizeof(struct sockaddr_in) ;

	if( *dataSocket > 1024 )
		*dataSocket=localHandle;
	nrReads = recvfrom( *dataSocket, message+4, msgSize, 0,
			    (struct sockaddr *)&remote, &addressSize ) ;
	if (nrReads < 0) return FAILED ;
	hostNr = ntohl( remote.sin_addr.s_addr ) ;
	*dataSocket = portNr = ntohs( remote.sin_port ) ;
 	(void) memcpy( (void *) remoteTAddress, (const void *)&hostNr,   4 );
	(void) memcpy( (void *) (remoteTAddress + 4), (const void *)&portNr,  2);
	return OK ;
	
} 	/* end of function SocketPacketReceive */

/*---------------------- end of file spmSocket.c -----------------------------*/

