/* --------------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:	spmCR.c
 *
 * Date:        May 1994
 *
 * Purpose:	This module implements the context resolver.
 *
 * Author( s):  Edward Veluwenkamp & Michel van Ooijen, Eric van Hengstum
 *
 * --------------------------------------------------------------------------
 */

#include <spmCR.h>

static char  V1_FILE[256] = "./v1.conf";

/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrInit
 *
 * Arguments: 	none
 *
 * Return:	STATUS, always OK.
 *
 * Purpose: 	This function sets some module constants, allocates the
 *		global datastructures and initializes them.
 *
 * Remarks: 	none
 *
 * -----------------------------------------------------------------------
 */
        STATUS
CrInit( )
{
	register unsigned short i;

	(void) memset( defaultVector, 0xFF, MAX_SUBS );
	(void) memset( vectorTable, 0x00, MAX_ROWS*MAX_SUBS );
	(void) memset( compareTable, 0x00, MAX_PARS*MAX_SUBS );
	for( i = 0; i < MAX_ROWS; i++ )
		indexTable[ i ].type = IND_EMPTY;
	for( i = 0; i < MAX_ROWS; i++ )
		aclCache[ i ].counter = 0;

	return( OK );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrOpen
 *
 * Arguments: 	none.
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function opens the CR module, meaning creates and
 *		initializes the global datastructures, for instance the 
 *		vectorTable.
 *
 * Remarks:  	none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrOpen( )
{
	static BOOLEAN init = FALSE;

	if( init )
		return FAILED;
	init = TRUE;

	if( CrInit( ) !=  OK )
		return FAILED;
	if( CrFill( vectorTable, indexTable, aclCache ) !=  OK )
		return FAILED;
	return OK;
}




/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrClose
 *
 * Arguments: 	none.
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function frees the allocated datastructures.
 *
 * Remarks: 	none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrClose( )
{
	/* if( CrFree( ) != OK )
		return FAILED; */
	return OK;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrMapPduVector
 *
 * Arguments: 	pduType, specifies type of pdu (IN).
 *		compareTable, selected vector table (IN/OUT)
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function maps pdutype upon vector.
 *
 * Remarks: 	none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrMapPduVector( PDUsChoiceId pduType, VECTORTABLE vectorTable, VECTOR compareVector )
{
	switch( (AsnInt) pduType ) {
        case PDUS_GET_REQUEST:
		memcpy( compareVector, vectorTable[GETVECTOR], MAX_SUBS );
		return OK;
	case PDUS_GET_NEXT_REQUEST:
		memcpy( compareVector, vectorTable[GETNEXTVECTOR], MAX_SUBS );
		return OK;
	case PDUS_RESPONSE:
		memcpy( compareVector, vectorTable[RESPONSEVECTOR], MAX_SUBS );
		return OK;
	case PDUS_SET_REQUEST:
		memcpy( compareVector, vectorTable[SETVECTOR], MAX_SUBS );
		return OK;
	case PDUS_GET_BULK_REQUEST:
		memcpy( compareVector, vectorTable[GETBULKVECTOR], MAX_SUBS );
		return OK;
	case PDUS_INFORM_REQUEST:
		memcpy( compareVector, vectorTable[INFORMVECTOR], MAX_SUBS );
		return OK;
	case PDUS_SNMPV2_TRAP:
		memcpy( compareVector, vectorTable[TRAPVECTOR], MAX_SUBS );
		return OK;
	}
	return FAILED;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrMapAuthVector
 *
 * Arguments: 	qos, specifies quality of service (IN).
 *		vectorTable, initial vectorTable (IN).
 *		compareVector, selected vector (IN/OUT)
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function maps authentication upon vector.
 *
 * Remarks: 	none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrMapAuthVector( QOS qos, VECTORTABLE vectorTable, VECTOR compareVector )
{
	switch( (AsnInt) qos ) {
	case qosNONE:
		memcpy( compareVector, vectorTable[NOAUTHVECTOR], MAX_SUBS );
		return OK;
	case qosAUTH:
	case qosPRIV:
		memcpy( compareVector, vectorTable[AUTHVECTOR], MAX_SUBS );
		return OK;
	}
	return FAILED;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrMapPrivVector
 *
 * Arguments: 	qos, specifies quality of service (IN).
 *		vectorTable, initial vectorTable (IN).
 *		compareVector, selected vector (IN/OUT)
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function maps privacy upon vector.
 *
 * Remarks: 	none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrMapPrivVector( QOS qos, VECTORTABLE vectorTable, VECTOR compareVector )
{
	switch( (AsnInt) qos ) {
	case qosNONE:
	case qosAUTH:
		memcpy( compareVector, vectorTable[NOPRIVVECTOR], MAX_SUBS );
		return OK;
	case qosPRIV:
		memcpy( compareVector, vectorTable[PRIVVECTOR], MAX_SUBS );
		return OK;
	}
	return FAILED;
}

/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrMapContextLocalVector
 *
 * Arguments: 	choice, specifies the choicee value (IN).
 *		vectorTable, initial vectorTable (IN).
 *		compareVector, selected vector (IN/OUT)
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function maps privacy upon vector.
 *
 * Remarks: 	none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrMapContextLocalVector( TruthValue choice, VECTORTABLE vectorTable, VECTOR compareVector )
{
	if (choice == true) {
		memcpy( compareVector, vectorTable[LOCALCONTEXTVECTOR], MAX_SUBS );
		return OK;
	}
	return FAILED;
}




/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrMapTimeVector
 *
 * Arguments: 	time, specifies contexLocalTime (IN).
 *		vectorTable, initial vectorTable (IN).
 *		compareVector, selected vector (IN/OUT)
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function maps contextLocalTime upon vector.
 *
 * Remarks: 	none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrMapTimeVector( AsnOid time, VECTORTABLE vectorTable, VECTOR compareVector )
{
	if ( CmpOID( &time, &currentTime ) == TRUE ) {
		memcpy(compareVector,vectorTable[CURRENTTIMEVECTOR],MAX_SUBS );
		return OK;
	} else if ( CmpOID( &time, &cacheTime ) == TRUE ) {
		memcpy(compareVector,vectorTable[CACHETIMEVECTOR],MAX_SUBS );
		return OK;
	} else if ( CmpOID( &time, &restartTime ) == TRUE ) {
		memcpy(compareVector,vectorTable[RESTARTTIMEVECTOR],MAX_SUBS );
		return OK;
	}else return FAILED;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrMapAddressVector
 *
 * Arguments: 	tAddress, specifies transport address of target (IN).
 *		vectorTable, initial vectorTable (IN).
 *		indexTable, indexes in the vectoTable (IN).
 *		compareVector, selected vector (IN/OUT)
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function maps transport address upon vector.
 *
 * Remarks: 	none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrMapAddressVector( AsnOcts *tAddress, VECTORTABLE vectorTable, 
		    INDEXTABLE indexTable, VECTOR compareVector, 
		    AsnInt *version, AsnOcts *community)
{
	register short i;
	AsnInt index;

	if( !tAddress->octetLen )
		return FAILED;

	for( i = 0; i < MAX_ROWS; i++ ) {
		if( ( indexTable[i].type == IND_ADDRESS ) &&
		    ( memcmp( indexTable[i].buf, tAddress->octs, 
			      tAddress->octetLen ) == 0 ) ) {
			*version = indexTable[i].version; 
			if (indexTable[i].version == SNMPVERSION1 ) {
				community->octetLen = indexTable[i].community->octetLen;
				community->octs = indexTable[i].community->octs;
				tAddress->octetLen = 6;
				(void) memcpy(tAddress->octs, indexTable[i].buf, 6) ; 
				return OK ;
			} else {
				index = indexTable[i].index;
				memcpy( compareVector, vectorTable[index], MAX_SUBS );
				community->octetLen = 0 ;
				community->octs = NULL; 
				return OK;
			} 	/* version */
		}		/* IND_ADDRESS */
	}			/* MAX _ROWS */ 
	return FAILED;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrMapEntityVector
 *
 * Arguments: 	localEntity, specifies contextLocalEntity (IN).
 *		vectorTable, initial vectorTable (IN).
 *		indexTable, indexes in the vectoTable (IN).
 *		compareVector, selected vector (IN/OUT)
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function maps contextLocalEntity upon vector.
 *
 * Remarks: 	none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrMapEntityVector( AsnOcts *localEntity, VECTORTABLE vectorTable, 
		    INDEXTABLE indexTable, VECTOR compareVector )
{
	register short i;
	AsnInt index;

	if( !localEntity->octetLen ) {
		memcpy( compareVector, defaultVector, MAX_SUBS );
		return OK;
	}

	for( i = 0; i < MAX_ROWS; i++ ) {
		if( ( indexTable[i].type == IND_ENTITY ) &&
		    ( memcmp( indexTable[i].buf, localEntity->octs, 
			      indexTable[i].length ) == 0 ) ) {
		      	index = indexTable[i].index;
			memcpy( compareVector, vectorTable[index], MAX_SUBS );
			return OK;
		}
	}
	return FAILED;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrMap
 *
 * Arguments: 	pduType, specifies type of pdu (IN).
 *		qos, quality of service (IN).
 *		localTime, contextLocalTime (IN).
 *		tAddress, transport address (IN).
 *		localEntity, contexLocalEntity (IN).
 *		srcId, source partyidentifier (OUT).
 *		dstId, destination party identifier (OUT).
 *		cxtId, context identifier (OUT).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function maps input parameters upon acl entry.
 *
 * Remarks: 	none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrMap( PDUsChoiceId pduType, QOS qos, AsnOid localTime, AsnOcts *tAddress,
       AsnOcts *localEntity, UAsnInt *srcId, UAsnInt *dstId, UAsnInt *cxtId,
       AsnInt *version, AsnOcts *community)
{
	register short i;

	(void) memset( compareTable, 0x00, MAX_PARS*MAX_SUBS );

	if( CrMapAddressVector( tAddress, vectorTable, indexTable, 
				compareTable[0], version, community) != OK )
		return FAILED;

	if (*version == SNMPVERSION1) {
		TRACE( SNMP_DEBUG_CR, "using SNMP v1 address and communnity") ;
		return OK ; 	/* v1 pdu, break and use community */
	}
	
	if( CrMapPduVector( pduType, vectorTable, compareTable[1] ) != OK )
		return FAILED;
	if( CrMapPrivVector( qos, vectorTable, compareTable[2] ) != OK )
		return FAILED;
	if( CrMapAuthVector( qos, vectorTable, compareTable[3] ) != OK )
		return FAILED;
	if( CrMapTimeVector( localTime, vectorTable, compareTable[4] ) != OK )
		return FAILED;
	if( CrMapEntityVector( localEntity, vectorTable, indexTable, 
			       compareTable[5] ) != OK )
		return FAILED;

	if( TRACE( SNMP_DEBUG_CR, "compareTable") == OK ) {
		(void) fprintf( stderr, "\n" );
		for( i = 0; i < MAX_PARS; i++ ) {
			(void) CrDebugVector( compareTable[i] );
			(void) fprintf( stderr, "\n" );
		} 
	}		

	if( CrCompare( compareTable, aclCache, MAX_PARS, srcId, dstId, cxtId ) != OK )
		return FAILED;
	return OK;
}

/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrMapTrap
 *
 * Arguments: 	notification, specifies type of trap (IN).
 *		srcId, source partyidentifier (OUT).
 *		dstId, destination party identifier (OUT).
 *		cxtId, context identifier (OUT).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function maps input parameters upon acl entry.
 *
 * Remarks: 	notification not yet supported. should be used to 
 * 		determine the appropriate view in corresponding to the context-
 *              ViewIndex. 
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrMapTrap( AsnOid *notification, UAsnInt *srcId, UAsnInt *dstId, UAsnInt *cxtId )
{
	register short i;

	(void) memset( compareTable, 0x00, MAX_PARS*MAX_SUBS );

	if( CrMapPduVector( PDUS_SNMPV2_TRAP, vectorTable, compareTable[0] ) != OK )
		return FAILED;
	if( CrMapContextLocalVector( true, vectorTable, compareTable[1] ) != OK )
		return FAILED;

	if( TRACE( SNMP_DEBUG_CR, "compareTable") == OK ) {
		(void) fprintf( stderr, "\n" );
		for( i = 0; i < MAX_PARS; i++ ) {
			(void) CrDebugVector( compareTable[i] );
			(void) fprintf( stderr, "\n" );
		} 
	}		

	if( CrCompare( compareTable, aclCache, 2, srcId, dstId, cxtId ) != OK )
		return FAILED;

	return OK;
}




/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrUnmap
 *
 * Arguments: 	
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	
 *
 * Remarks: 	none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrUnmap( )
{
	return( OK );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrDebug
 *
 * Arguments: 	vectorTable, list of initial vectors (IN).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function prAsnInts the initial vectorTable.
 *
 * Remarks: 	none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrDebugBit( short bit )
{
	if( bit )
		(void) fprintf( stderr, "1" );
	else
		(void) fprintf( stderr, "." );
	return OK;
}

	STATUS
CrDebugSubVector( CELL subVector )
{
	register short i;

	for( i = MAX_BITS-1; i >= 0; i-- ) 
		(void) CrDebugBit( (1 << i) & subVector );
	return OK;
}

	STATUS
CrDebugVector( VECTOR vector )
{
	register unsigned short	i;

	for( i = 0; i < MAX_SUBS; i++ ) { 
		(void) CrDebugSubVector( vector[i] );
		(void) fprintf( stderr, " " );
	}
	return OK;
}

	STATUS
CrDebugVectorTable( VECTORTABLE vectorTable )
{
	register unsigned short	i;

	for( i = 0; i < MAX_ROWS; i++ ) { 
		(void) CrDebugVector( vectorTable[i] );
		(void) fprintf( stderr, "[%2d]\n", i);
	}
	return OK;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrMapBit 
 *
 * Arguments: 	aclNr, aclentry number (IN)
 *          	subVectorNr, number of subvector within vector (OUT)
 *          	bitNr, number if bit within subvector (OUT)
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function maps an aclentry upon bitposition in vector.
 *
 * Remarks: 	none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrMapBit( AsnInt aclNr, AsnInt *subVectorNr, AsnInt *bitNr )
{
	*subVectorNr = aclNr / MAX_BITS;
	*bitNr = ( 1 << ( ( MAX_BITS - 1 ) - ( aclNr % MAX_BITS ) ) );
	return OK;
}

 

/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrSetBit 
 *
 * Arguments: 	subVectorNr, number of subvector in vector (IN).
 *          	bitNr, number of bit in subvector (IN).
 *          	vector, the initial vector (IN/OUT).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function sets bit at the selected position in vector.
 *
 * Remarks: 	none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrSetBit( AsnInt subVectorNr, AsnInt bitNr, VECTOR vector )
{
	vector[subVectorNr] = vector[subVectorNr] | (bitNr);
	return OK;
}


  
/* 
 * -----------------------------------------------------------------------
 *
 * Function:  	Crcomparevectors 
 *
 * Arguments: 	vector1, first vector (IN).
 *          	vector2, second vector (IN).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function ands two vectors. 
 *
 * Remarks: 	none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrCompare( COMPARETABLE compareTable, ACLCACHE aclCache, int numberOfVectors,
	   UAsnInt *srcId, UAsnInt *dstId, UAsnInt *cxtId )
{
	register short aclNr, parNr;
	AsnInt subVectorNr, bitNr;

	for( aclNr = 0; aclNr < MAX_SUBS*MAX_BITS; aclNr++ ) {
		CrMapBit( aclNr, &subVectorNr, &bitNr );
		for( parNr = 0; parNr < numberOfVectors; parNr++ ) {
			if( !( compareTable[parNr][subVectorNr] & bitNr ) )
				break;
		}
		if( parNr == numberOfVectors ) {
			*srcId = aclCache[aclNr].subject;
			*dstId = aclCache[aclNr].target;
			*cxtId = aclCache[aclNr].resources;
			return OK;
		}
	}
	return FAILED;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrFillPduVector
 *
 * Arguments: 	privileges, specifies the acl priviledges (IN).
 *		subVectorNr, identifies the subvector (IN).
 *		bitNr, identifies the bit within subvector (IN).
 *		vectorTable, list of initial vectors (IN/OUT).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function checks the specified acl priviledges against
 *		pdutypes.
 *
 * Remarks:	none 
 *
 * -----------------------------------------------------------------------
 */
	STATUS 
CrFillPduVector( AsnInt privileges, 
		 AsnInt subVectorNr, AsnInt bitNr, VECTORTABLE vectorTable )
{
	if( privileges & 0x1 )
		CrSetBit( subVectorNr, bitNr, vectorTable[GETVECTOR] );
	if( privileges & 0x2 )
		CrSetBit( subVectorNr, bitNr, vectorTable[GETNEXTVECTOR] );
	if( privileges & 0x4 )
		CrSetBit( subVectorNr, bitNr, vectorTable[RESPONSEVECTOR] );
	if( privileges & 0x8 )
		CrSetBit( subVectorNr, bitNr, vectorTable[SETVECTOR] );
	if( privileges & 0x20 )
		CrSetBit( subVectorNr, bitNr, vectorTable[GETBULKVECTOR] );
	if( privileges & 0x40 )
		CrSetBit( subVectorNr, bitNr, vectorTable[INFORMVECTOR] );
	if( privileges & 0x80 )
   		CrSetBit( subVectorNr, bitNr, vectorTable[TRAPVECTOR] );
	return OK ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrFillAuthVector
 *
 * Arguments: 	authProtocol, specifies the authentication protocol (IN).
 *		subVectorNr, identifies the subvector (IN).
 *		bitNr, identifies the bit within subvector (IN).
 *		vectorTable, list of initial vectors (IN/OUT).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function checks the acl target support for aurthentication
 *		protocol.
 *
 * Remarks:	none 
 *
 * -----------------------------------------------------------------------
 */
	STATUS 
CrFillAuthVector( AsnOid *authProtocol,
		   AsnInt subVectorNr, AsnInt bitNr, VECTORTABLE vectorTable )
{
	if( CmpOID( authProtocol, &noAuth ) == TRUE )
		CrSetBit( subVectorNr, bitNr, vectorTable[NOAUTHVECTOR] );
	else if( CmpOID( authProtocol, &v2md5AuthProtocol ) == TRUE )
		CrSetBit( subVectorNr, bitNr, vectorTable[AUTHVECTOR] );
	return OK;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrFillPrivVector
 *
 * Arguments: 	privProtocol, specifies the privicy protocol (IN).
 *		subVectorNr, identifies the subvector (IN).
 *		bitNr, identifies the bit within subvector (IN).
 *		vectorTable, list of initial vectors (IN/OUT).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function checks the the specified privicy protocol
 *
 * Remarks: 
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrFillPrivVector( AsnOid *privprotocol,
		   AsnInt subVectorNr, AsnInt bitNr, VECTORTABLE vectorTable )
{
	if ( CmpOID( privprotocol, &noPriv ) == TRUE )
		CrSetBit( subVectorNr, bitNr, vectorTable[NOPRIVVECTOR] );
	else if ( CmpOID( privprotocol, &desPrivProtocol ) == TRUE )
		CrSetBit( subVectorNr, bitNr, vectorTable[PRIVVECTOR] );
	return OK;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrFillContextLocalVector
 *
 * Arguments: 	contextLocal, true or false (IN).
 *		subVectorNr, identifies the subvector (IN).
 *		bitNr, identifies the bit within subvector (IN).
 *		vectorTable, list of initial vectors (IN/OUT).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function checks if the context is locally supported.
 *
 * Remarks: 
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrFillContextLocalVector( TruthValue contextLocal, 
		   AsnInt subVectorNr, AsnInt bitNr, VECTORTABLE vectorTable )
{
	if ( contextLocal == true )
		CrSetBit( subVectorNr, bitNr, vectorTable[LOCALCONTEXTVECTOR] );
	return OK ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrFillTimeVector
 *
 * Arguments: 	contextLocalTime, specifies RESTART or CURRENTTIME (IN)
 *		subVectorNr, identifies the subvector (IN).
 *		bitNr, identifies the bit within subvector (IN).
 *		vectorTable, list of initial vectors (IN/OUT).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function checks the the specified privicy protocol
 *
 * Remarks: 	none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrFillTimeVector( AsnOid *contextLocalTime,
		   AsnInt subVectorNr, AsnInt bitNr, VECTORTABLE vectorTable )
{
	if ( CmpOID( contextLocalTime, &currentTime ) == TRUE ) {
		CrSetBit( subVectorNr, bitNr, vectorTable[CURRENTTIMEVECTOR] );
		return OK ;
	} else if ( CmpOID( contextLocalTime, &restartTime ) == TRUE ) {
		CrSetBit( subVectorNr, bitNr, vectorTable[RESTARTTIMEVECTOR] );
		return OK ;
	} else if ( CmpOID( contextLocalTime, &cacheTime ) == TRUE ) {
		CrSetBit( subVectorNr, bitNr, vectorTable[CACHETIMEVECTOR] );
		return OK ;
	}
	return OK;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrFillEntityVector
 *
 * Arguments: 	contextLocalEntity, specifies name of target (IN).
 *		subVectorNr, identifies the subvector (IN).
 *		bitNr, identifies the bit within subvector (IN).
 *		vectorTable, list of initial vectors (IN/OUT).
 *		indexTable, indexes in initial vectors-list (IN/OUT).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function checks the name of agent.
 *
 * Remarks: 
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrFillEntityVector( AsnOcts *contextLocalEntity, AsnInt subVectorNr, AsnInt bitNr,
	             VECTORTABLE vectorTable, INDEXTABLE indexTable )
{
	AsnInt i, index = CACHETIMEVECTOR + 1;

	if( ! contextLocalEntity->octetLen )
		return FAILED;

	for( i = 0; i < MAX_ROWS; i++ ) {
		if( indexTable[i].type == IND_EMPTY ) {
			indexTable[i].type = IND_ENTITY;
			indexTable[i].index = index;
			indexTable[i].length = contextLocalEntity->octetLen;
			memcpy( indexTable[i].buf, contextLocalEntity->octs,
				contextLocalEntity->octetLen );
			CrSetBit( subVectorNr, bitNr, vectorTable[index] );
			return OK;
		}
		if( indexTable[i].type != IND_EMPTY )
			index = indexTable[i].index + 1;
		if( ( indexTable[i].type == IND_ENTITY ) &&
		    ( memcmp( indexTable[i].buf, contextLocalEntity->octs, 
			      indexTable[i].length ) == 0 ) ) {
			CrSetBit( subVectorNr, bitNr, vectorTable[indexTable[i].index] );
			return OK;
		}
	}
	return FAILED;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrFillAddressVector
 *
 * Arguments: 	tAddress, specifies transport address (IN).
 *		subVectorNr, identifies the subvector (IN).
 *		bitNr, identifies the bit within subvector (IN).
 *		vectorTable, list of initial vectors (IN/OUT).
 *		indexTable, indexes in initial vectors-list (IN/OUT).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function checks address of target.
 *
 * Remarks: 
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrFillAddressVector( AsnOcts *tAddress, AsnInt subVectorNr, AsnInt bitNr, 
		     VECTORTABLE vectorTable, INDEXTABLE indexTable )
{
	AsnInt i, index = CACHETIMEVECTOR + 1;
	short port;

	memcpy( &port, tAddress->octs+4, 2 );
	TRACE( SNMP_DEBUG_CR, "tAddress=%u.%u.%u.%u.%d", tAddress->octs[0], 
			tAddress->octs[1], tAddress->octs[2], tAddress->octs[3],
			port );
           
	if( !(tAddress->octetLen) )
		return FAILED;

	for( i = 0; i < MAX_ROWS; i++ ) {
		if( indexTable[i].type == IND_EMPTY ) {
			indexTable[i].type = IND_ADDRESS;
			indexTable[i].index = index;
			indexTable[i].length = tAddress->octetLen;
			memcpy( indexTable[i].buf, tAddress->octs, tAddress->octetLen );
			CrSetBit( subVectorNr, bitNr, vectorTable[index] );
			indexTable[i].version = SNMPVERSION2 ;
			return OK;
		} else 
			index = indexTable[i].index + 1;

		if( ( indexTable[i].type == IND_ADDRESS ) &&
		    ( memcmp( indexTable[i].buf, tAddress->octs, 
			      4 /*indexTable[i].length */ ) == 0 ) ) {
			CrSetBit( subVectorNr, bitNr, vectorTable[indexTable[i].index] );
			return OK;
		}
	}
	return FAILED;
}

/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrFillAddressVector
 *
 * Arguments: 	tAddress, specifies transport address (IN).
 *		subVectorNr, identifies the subvector (IN).
 *		bitNr, identifies the bit within subvector (IN).
 *		vectorTable, list of initial vectors (IN/OUT).
 *		indexTable, indexes in initial vectors-list (IN/OUT).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function checks address of target.
 *
 * Remarks: 
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CrFillV1Module()
{
	FILE    *fp=NULL;
	char    *cp=NULL;
	int     blank;          /* boolean to skip blank lines */ 
	int     communitySize;
	int     BUFSIZE=254; 
	char    buf[BUFSIZE], buf1[BUFSIZE], buf2[BUFSIZE], buf3[BUFSIZE];
	u_long  addr;
	short   port;

	AsnInt i, index;
	AsnOcts *community; 

	cp = getenv("UT_SNMP_HOME");
	if( cp != (char *) NULL ) {
		strcpy(V1_FILE, cp);
		strcat(V1_FILE, "/etc/v1.conf" );
	}

	if( OpenPartyFile( V1_FILE, "r", &fp ) != OK ) {
		Error( "Error reading file %s", V1_FILE );
		return ERROR ;
	} 


	while( fgets( buf, BUFSIZE, fp ) ) {
	if( strlen(buf) > BUFSIZE ) {
		Error( "Line longer than %d bytes", BUFSIZE);
		continue ;
	}
	if( buf[0] == '#' ) continue;
	blank = TRUE;
	for( cp = buf; *cp; cp++ )
		if( !isspace( *cp ) ) {
			blank = FALSE;
			break;
		}
		if( blank ) continue;

		memset( buf1, 0, (size_t) BUFSIZE );
		memset( buf2, 0, (size_t) BUFSIZE );
		memset( buf3, 0, (size_t) BUFSIZE );

		if( sscanf( buf, "%s %s %s", buf1, buf2, buf3) != 3) {
			Error( "Bad parse" );
			continue ;
		}

		/* find first empty entry */ 
  
		for( i = 0; i < MAX_ROWS; i++ ) {
			if( indexTable[i].type != IND_EMPTY )
				index = indexTable[i].index + 1;
			else 
				break;	/* break of FOR loop */
		} 	

		if( indexTable[i].type != IND_EMPTY )
			continue;

		indexTable[i].type = IND_ADDRESS;

		communitySize = strlen(buf1) ;
		community = pNewOct(communitySize) ;
		community->octetLen = communitySize ;
		memcpy( community->octs, &buf1, communitySize );
		indexTable[i].version = SNMPVERSION1 ;
		indexTable[i].community = community ;

		if( (addr = inet_addr( buf2 )) == -1 ) {
			Error( "Bad IP address" );
			pFreeOct(community);
			continue; 
		}         
		addr = htonl(addr);
		indexTable[i].length = 6;
		memcpy( indexTable[i].buf, &addr, 4 );

		for( cp = buf3; *cp; cp++ )
			if( !isdigit( *cp ) ) {
				Error( "Not a port number" );
				pFreeOct(community); 
				continue; 
			}
		port = htons(atoi(buf3));
		memcpy( indexTable[i].buf + 4, &port, 2 );

		TRACE(SNMP_DEBUG_CR,"%s \tv1 address=%u.%u.%u.%u.%d);\n", 
		community->octs, indexTable[i].buf[0], indexTable[i].buf[1], 
		indexTable[i].buf[2], indexTable[i].buf[3], port );
		if (TransportOpen (0, indexTable[i].buf) !=OK){
			Error("Open failed at address %u.%u.%u.%u.%d",
				indexTable[i].buf[0], indexTable[i].buf[1],
				indexTable[i].buf[2], indexTable[i].buf[3], port);
			break;
		}
	}       /* fgets */

	return OK ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	CrFill 
 *
 * Arguments: 	vectorTable, list of initial vectors (IN/OUT).
 *		indexTable, maps tAddress v contextLocalEntity on index of
 *			    initial vector (IN/OUT).
 *		aclCache, AsnInternal acl list (IN/OUT).
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function fills the initial vectors.
 *
 * Remarks:	none  
 *
 * -----------------------------------------------------------------------
 */
	STATUS 
CrFill( VECTORTABLE vectorTable, INDEXTABLE indexTable, ACLCACHE aclCache )
{
	AsnInt		privileges, aclNr, subVectorNr, bitNr;
	TruthValue	contextLocal; 
	AsnOid		*authProtocol, *privProtocol, *contextLocalTime ;
	AsnOcts 	*contextLocalEntity, *tAddress;


	if( MibResetAclEntry() != OK )
		return FAILED ;

	authProtocol = pNewOid( 9 ) ; 		/* "\53\6\1\6\3\3\1\1\?" = 9 oids	*/
	privProtocol = pNewOid( 9 ) ; 		/* "\53\6\1\6\3\3\1\1\?" = 9 oids	*/
	contextLocalTime = pNewOid( 9 ) ; 	/* "\53\6\1\6\3\3\1\2\?" = 9 oids	*/
	contextLocalEntity = pNewOct( 256 );	/* maximum length octetstring 	*/	
	tAddress = pNewOct( 256 );		

	for( 	aclNr = 0;
		MibGetAclEntry( &aclCache[aclNr].subject, &aclCache[aclNr].target,
			        &aclCache[aclNr].resources, &privileges ) == OK;
		aclNr++ ) 
	{
		aclCache[aclNr].counter = aclNr;
		CrMapBit( aclNr, &subVectorNr, &bitNr );
		CrFillPduVector( privileges, subVectorNr, bitNr, vectorTable ); 
		if( MibGetPartyAuthProtocol( aclCache[aclNr].target, 
					     authProtocol ) == OK )
			CrFillAuthVector( authProtocol, subVectorNr, bitNr, vectorTable );
		if( MibGetPartyPrivProtocol( aclCache[aclNr].target, 
					     privProtocol ) == OK )
			CrFillPrivVector( privProtocol, subVectorNr, bitNr, vectorTable );
		if( MibGetContextLocal( aclCache[aclNr].resources,
					    &contextLocal ) == OK ) 
			CrFillContextLocalVector( contextLocal, subVectorNr, bitNr, vectorTable );
		if( MibGetContextLocalTime( aclCache[aclNr].resources,
					    contextLocalTime ) == OK ) 
			CrFillTimeVector( contextLocalTime, subVectorNr, bitNr,
					   vectorTable );
		if( MibGetContextLocalEntity( aclCache[aclNr].resources, 
					      contextLocalEntity ) == OK )
			CrFillEntityVector( contextLocalEntity, subVectorNr, bitNr,
					     vectorTable, indexTable );
		if( MibGetPartyTAddress( aclCache[aclNr].target, tAddress ) == OK )
			CrFillAddressVector( tAddress, subVectorNr, bitNr ,
					      vectorTable, indexTable );
	}

	(void) CrFillV1Module() ;

	pFreeOid( authProtocol ); pFreeOid( privProtocol ); 
	pFreeOid( contextLocalTime );  
	pFreeOct( contextLocalEntity ); pFreeOct( tAddress );		

	if( TRACE( SNMP_DEBUG_CR, "CrDebugVectorTable" ) == OK )
		(void) CrDebugVectorTable( vectorTable );

	return OK;
}
   


/* -----------------------------------------------------------------------
 */

