/* -----------------------------------------------------------------------
 *
 * Project:     UT-SNMP
 *
 * Filename:    per_int.c
 *
 * Date:        May 1994.
 *
 * Purpose:     Interface to PER modules.
 *
 * Author(s):   Rene Oude Vrielink
 *
 * -----------------------------------------------------------------------
 */

#include <snmpTypes.h>
#include <snacc/asn_config.h>
#include <snacc/asn_incl.h>
#include <perv2/per_buf.h>
#include <perv2/per_int.h>


/* -------------------------------------------------------------
 * Function:	PerEncInt
 * Arguments:	intvalue, AsnInt ,(IN)
 *		b, ...., (OUT) (unsigned char **)	
 * Purpose:	Encodes an unsigned integer into PER positive
 *		binary intger with as few octets as possible
 * --------------------------------------------------------------
 */

	AsnLen	
PerEncInt(BUF_TYPE  b, AsnInt *intvalue)
{
  int             len;
  int             i;
  unsigned long int   mask;
  unsigned long int   dataCpy;

#define INT_MASK (0xff00 << ((sizeof(AsnInt) - 2) * 8))
#define SIGN_MASK (0x8000 << ((sizeof(AsnInt) - 2) * 8))

  dataCpy = *intvalue;

  mask = INT_MASK; /* the mask is used to check every first 8 bits
		    * this way you be sure to use as few octets 
		    * as possible.
		    */

  if ((dataCpy & SIGN_MASK) == 0)   /* this if statment switches the MSBit of dataCpy */
       dataCpy |= SIGN_MASK;
  else
       dataCpy &= ~(SIGN_MASK);

  for (len = sizeof(AsnInt) ; len > 1 ; --len)
   {
 	if ( (dataCpy & mask) == 0)
	      mask >>= 8;
	else
      	break;
   }

   /*
    * write the PER integer
    */
  for (i = 0; i < len; i++)
  {
   	PerPutByte(b ,dataCpy);
	dataCpy >>= 8;
  }

  return(len); /* len is total number of octets needed */
} /* PerEncInt */

/* ----------------------------------------------------------------------
 * Function:	PerEncUInt
 * Arguments:	uintvalue, UAsnInt ,(IN)
 *		b, ....., (OUT)	
 * Returns:	length of the encoding
 * Purpose:	Encodes an unsigned integer into PER positive
 *		binary intger with as few octets as possible
 * --------------------------------------------------------------
 */

	AsnLen	
PerEncUInt(BUF_TYPE b, UAsnInt *uintvalue)
{
  int             len;
  int             i;
  unsigned long int   mask;
  unsigned long int   dataCpy;

  dataCpy = *uintvalue;

  /*
   * calculate encoded length of the integer
   */

  mask = INT_MASK;
  for (len = sizeof(UAsnInt) ; len > 1 ; --len)
   {
 	if ( (dataCpy & mask) == 0)
	      mask >>= 8;
	else
      	break;
   }
  
  for (i = 0; i < len; i++)
  {
   	PerPutByte(b, dataCpy);
	dataCpy >>= 8;
  }
  return(len); /* len is total number of octets needed */
}
	
/* ----------------------------------------------------------------
 * Function:	PerDecUInt
 * Arguments:	len ,.... , (IN)
 * 		b,	....,(IN)
 *		uintvalue, ..... , (OUT)
 * Returns:	ERROR or OK
 * Purpose:	Decodes a positive binary integer in PER format
 *		into an unsigned long integer. 
 * Remarks:	The lenght of the PER format is len (in octets)	
 * -----------------------------------------------------------------
 */
	STATUS
PerDecUInt(BUF_TYPE b, UAsnInt *uintvalue, UAsnInt len)
{
  unsigned long int	retVal = 0;
  int             	i;

  
  for (i = 0; i < len; i++)
   	retVal = (retVal << 8) | (unsigned long int) PerGetByte(b);
  
  *uintvalue = (UAsnInt) retVal;
  return( OK ); 
}
/* ----------------------------------------------------------------
 * Function:	PerDecInt
 * Arguments:	intvalue, ..... , (OUT)
 * 		b,	....,(IN)
 *		len, ...., (IN)
 * Returns:	ERROR or OK
 * Purpose:	Decodes a positive binary integer in PER format
 *		into an signed long integer.
 * Remarks:	The lenght of the PER format is len (in octets)	
 * -----------------------------------------------------------------
 */
	STATUS
PerDecInt(BUF_TYPE b, AsnInt *intvalue, UAsnInt len)
{
  unsigned long int	retVal = 0;
  int             	i;


  for (i = 0; i < len; i++)
   	retVal = (retVal << 8) | (unsigned long int) PerGetByte(b);
 
  /* this if statment switches the MSBit of dataCpy 
   * this way you change a PER integer to signed integer
   * e.g. zero becomes -(2^31) 
   */
  if ((retVal & SIGN_MASK) == 0)   
       retVal |= SIGN_MASK;
  else
       retVal &= ~(SIGN_MASK);
 
  *intvalue = (AsnInt) retVal;

  return( OK ); 
}

/*
 * PER encodes signed long integer's contents
 */
AsnLen
PerEncSInt(BUF_TYPE b, AsnInt* data)
{

    int             len;
    int             i;
    unsigned long int   mask;
    unsigned long int   dataCpy;

#define SINT_MASK (0x7f80 << ((sizeof(AsnInt) - 2) * 8))

    dataCpy = *data;

    /*
     * calculate encoded length of the integer (content)
     */
    mask = SINT_MASK;
    if ((long int)dataCpy < 0)
        for (len = sizeof(AsnInt) ; len > 1 ; --len)
        {
            if ( (dataCpy & mask) == mask)
                mask >>= 8;
            else
                break;
        }
    else
        for (len = sizeof(AsnInt) ; len > 1 ; --len)
        {
            if ( (dataCpy & mask) == 0)
                mask >>= 8;
            else
                break;
        }

    /*
     * write the BER integer
     */
    for (i = 0; i < len; i++)
    {
        PerPutByte(b, dataCpy);
        dataCpy >>= 8;
    }

    return(len);

}  /* PerEncSInt */

/*
 * PER Decodes a signed integer 
 */
STATUS
PerDecSInt(BUF_TYPE b, AsnInt *result, AsnLen len)
{
    int   i;
    long int   retVal;
    unsigned long int  byte;


    if ( len > sizeof(AsnInt))
    {
        Asn1Error(" ERROR - integer to big to decode.\n");       
        return(ERROR);
   }

    /*
     * look at integer value
     */
    byte =  (unsigned long int) PerGetByte(b);

    if (byte & 0x80)   /* top bit of first byte is sign bit */
        retVal = (-1 << 8) | byte;
    else
        retVal = byte;

    /*
     * write from buffer into long int
     */
    for (i = 1; i < len; i++)
        retVal = (retVal << 8) | (unsigned long int)(PerGetByte(b));

    *result = retVal;

}  /* PerDecSInt */
