/* -----------------------------------------------------------------------
 *
 * Project:     UT-SNMP
 *
 * Filename:    spmReceive.c
 *
 * Date:        May 1994
 *
 * Purpose:     Receiving and processing an incoming message. 
 *
 * Author(s):   Vincent Berkhout, Frederic P. H. van Hengstum
 *
 * -----------------------------------------------------------------------
 */
#include <spmReceive.h> 



/* 
 * -------------------------------------------------------------------
 *
 * Function:	SpmInitReceive	
 *
 * Arguments:	none 
 *
 * Returns:	STATUS,	which is OK on succes
 *
 * Purpose:	This function initializes the receive module.
 *		(opens all "listen endpoints") 
 *
 * Remark:
 *
 * -------------------------------------------------------------------
 */
	STATUS
SpmInitReceive()
{
	AsnOid	*domain=NULL ;
	AsnOcts	*address=NULL;
	UAsnInt	src=0, dst=0, ctx=0 ; 
	AsnInt	acl=0, local=0;
	TDOMAIN	d=UDPDOMAIN ;

	domain = pNewOid( 9 ) ;
	address = pNewOct( 6 ) ;

	/*
	 *	Open transport ports ...
	 */
	if (MibResetAclEntry() == OK ) 
		while ( MibGetAclEntry( &src, &dst, &ctx, &acl ) == OK ) {

			if( ( MibGetPartyLocal( src, (TruthValue *) &local ) != OK ) || 
	    		    ( local != true ) ) continue ;
			if( MibGetPartyTDomain( src, domain ) != OK ) continue ;
			if( DomainConvert( domain, &d ) != OK ) continue ; 
			if( MibGetPartyTAddress( src, address ) != OK ) continue ;
			if( TransportOpen( d, address->octs ) != OK ) continue ;
		}

	pFreeOid(domain) ;
	pFreeOct(address) ;	
	return OK;
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:	SpmReceive	
 *
 * Arguments:	none
 *
 * Returns:	STATUS which is OK on succes
 *
 * Purpose:	This function processes an COMMUNICATION MESSAGE to be
 *		received.
 *
 * Remarks:	in the transport reveive call the local variable is NOT 
 * 		used.
 * 		
 * -------------------------------------------------------------------
 */
	STATUS
SpmReceive( ) 
{
	TMESSAGE	tmessage ;
	AsnOcts 	*msg=NULL, *remote=NULL, *local=NULL;
	UAsnInt		src, dst, ctx ;
	AsnInt 		reqId=0, errStat=0, errInd=0, tag=0 ;
	VarBindList 	*varBinds=NULL ; 
	TDOMAIN		domain = UDPDOMAIN ;

	AsnInt 		version, generic_trap, specific_trap; 
	UAsnInt		time_stamp;
	AsnOcts 	*community, *enterprise, *agent_addr; 

	/* 	CREATE "VOLATILE" MEMORY FOR DECODING THE MESSAGE
	 */ 
	InitNibbleMem( 64*1024, 512 );

	remote = NewOct(6);
	local = NewOct(6);
	local->octetLen = remote->octetLen = 6; 

	/*	RECEIVE COMMUNICATION MESSAGE FROM NETWORK.
	 */
	if( TransportReceive( &domain, local->octs, remote->octs, &tmessage ) 
		!= OK ) {
		ShutdownNibbleMem();
		return FAILED ;
	}

	msg = NewOct( 0 ) ;
	msg->octetLen = tmessage.length ;
	msg->octs = (char *) tmessage.buf ;

	reqId = errStat = errInd = tag = 0 ;

	/*	DECODE THE V1 OR V2 COMMUNICATION MESSAGE. 
	 */
	if ( *(msg->octs) == 0xA1 ) {			/* SNMPv2 message? */
		if( SpmUnwrapV2Msg( msg, &src, &dst, &ctx, &tag, &reqId,
				&errStat, &errInd, &varBinds ) != OK ) {
			ShutdownNibbleMem();
			return FAILED ;
		}
		version = SNMPVERSION2 ;

	} else if  (*(msg->octs) == 0x30 ) {		/* SNMPv1 message? */

		if ( SpmUnwrapV1Msg( msg, &tag, &reqId, &errStat, &errInd, &varBinds,
				&version, community, enterprise, agent_addr, 
				&generic_trap, &specific_trap, &time_stamp) != OK )  {
			ShutdownNibbleMem();
			return FAILED ;
		}
		src = dst = ctx = 0;
		version = SNMPVERSION1 ;
	} else {					/* unknown message type! */
		ShutdownNibbleMem();
		return FAILED;		/* This is where PER should be called! */
	}

	switch ( tag ) {

	case PDUS_SET_REQUEST :
	case PDUS_GET_REQUEST :
	case PDUS_GET_NEXT_REQUEST :
	case PDUS_GET_BULK_REQUEST :
		/*	ADMINISTRATE THE COMMUNICATION MESSAGE.
		 */
		if( MibInsertLocalEntry( reqId, src, dst, ctx, remote, domain ) != OK ) {
			ShutdownNibbleMem() ; 
			return FAILED;	
		}
		break ;

	case PDUS_INFORM_REQUEST :
		/* 	attention:
		 * 	inform is provider confirmed: swap src and dst!!!
		 */
                if( SpmWrapV2Message( dst, src, ctx, PDUS_RESPONSE, reqId, errStat, 
					errInd, varBinds, &msg ) != OK ) {
                        ShutdownNibbleMem( );
                        return FAILED ;
                }
		if( TransportSend( domain, local->octs, remote->octs, msg ) != OK ) {
			ShutdownNibbleMem( );
			return FAILED ;
		}
		break ; 

	case PDUS_RESPONSE :
	case PDUS_SNMPV2_TRAP : 
		break ;	/* do nothing */ 

	default: {
		ShutdownNibbleMem() ; 
		return FAILED ;
		}
	} 		/* tag */ 


	/*	SEND COMMUNICATION MESSAGE INFO TO APPLICATION
	 */
	if( AimSend( version, src, dst, ctx, tag, reqId, errStat, errInd, 
		     varBinds ) != OK ) {
		ShutdownNibbleMem() ; 
		return FAILED ;
	} 

	ShutdownNibbleMem() ; 
	return OK ;

}
